@@ -18,7 +18,7 @@ use File::Path ();
 use Cwd ();
 use Config;
 
-our $VERSION = '0.294';
+our $VERSION = '0.295';
 
 use constant WIN32 => $^O eq 'MSWin32';
 
@@ -71,6 +71,7 @@ sub parse_options {
     $self->{color} = 1 if !defined $self->{color} && -t STDOUT;
     if ($target_perl) {
         die "--target-perl option conflicts with --global option\n" if $self->{global};
+        die "--target-perl option can be used only if perl version >= 5.16.0\n" if $] < 5.016;
         # 5.8 is interpreted as 5.800, fix it
         $target_perl = "v$target_perl" if $target_perl =~ /^5\.[1-9]\d*$/;
         $self->{target_perl} = App::cpm::version->parse($target_perl)->numify;
@@ -181,7 +182,7 @@ sub cmd_install {
         inc    => $self->_inc,
         (exists $self->{target_perl} ? (target_perl => $self->{target_perl}) : ()),
     );
-    $self->register_initial_job($master);
+    $self->register_initial_job($master) or return 0;
 
     my $worker = App::cpm::Worker->new(
         verbose         => $self->{verbose},
@@ -291,7 +292,7 @@ sub register_initial_job {
         my ($is_satisfied, @need_resolve) = $master->is_satisfied($requirements);
         if (!@$dist and $is_satisfied) {
             warn "All requirements are satisfied.\n";
-            exit 0;
+            return 0;
         } elsif (!defined $is_satisfied) {
             my ($req) = grep { $_->{package} eq "perl" } @$requirements;
             die sprintf "%s requires perl %s\n", $self->{cpanfile}, $req->{version_range};
@@ -308,6 +309,7 @@ sub register_initial_job {
             dev => $p->{dev},
         );
     }
+    return 1;
 }
 
 sub load_cpanfile {
