@@ -1,7 +1,7 @@
 package App::cpm::Tutorial;
 use strict;
 use warnings;
-our $VERSION = '0.294';
+our $VERSION = '0.295';
 1;
 __END__
 
@@ -28,7 +28,7 @@ If not, then
 
   $ curl -sL http://cpanmin.us | perl - -nq App::cpm
 
-Moreover if you use perl 5.16+, then you can fetch I<self-contained> cpm.
+Moreover if you use perl 5.10.1+, then you can fetch I<self-contained> cpm.
 
   $ curl -sL https://git.io/cpm > cpm
 
